#!/usr/bin/python

import sys

# --------------------------------------------------------
# Defining some constants used in referring to the fields
# --------------------------------------------------------
ID = 0
PROD = 1
BRAND = 2

# --------------------------------------------------------
# Define suffix
# --------------------------------------------------------
SUFFIXES = ["Inc.", "Corp.", "Incorporation", "Corporation", "Technology", "Ltd.", "Limited"]


# --------------------------------------------------------
# Merge the dictionary with brands in the development set
# --------------------------------------------------------
def merge(dict, data):
    for line in data:
        if line[BRAND] not in dict:
            dict.append(line[BRAND])
    return


# --------------------------------------------------------
# Extract brand from the test set
# --------------------------------------------------------
def get_truth(test):
    truth = list()
    for rec in test:
        truth.append(rec[BRAND])
    return truth


def get_id(test):
    id = list()
    for rec in test:
        id.append(rec[ID])
    return id


# ---------------------------------------------------------
# Check if predicted brand name is subset of truth
# ---------------------------------------------------------
def check_subset(truth, extracted):
    for suffix in SUFFIXES:
        if (extracted.lower() + " " + suffix.lower() == truth.lower()) or \
                (truth.lower() + " " + suffix.lower() == extracted.lower()):
            return True

    # # case of partial match
    # truth_tokens = truth.lower().split(" ")
    # extracted_tokens = extracted.lower().split(" ")
    #
    # for token in truth_tokens:
    #     if token in extracted_tokens:
    #         return True

    return False


# --------------------------------------------------------
# Computer precision and recall from the truth and
# extracted brand names.
# --------------------------------------------------------
def compute_pr(id, truth, extracted):
    true_positive, true_negative = 0, 0
    false_positive, false_negative = 0, 0

    for i, j, k in zip(truth, extracted, id):

        if i.strip() == "" and j.strip() == "":
            true_negative += 1
        elif i.strip().lower() == j.strip().lower(): #or check_subset(i.strip(), j.strip()):
            print "tp, brand: " + i.strip() + ", " + "\t extracted: " + j.strip() + "\t id: " + k
            true_positive += 1
        elif j.strip() == "":
            false_negative += 1
            print "fn, brand: " + i.strip() + ", " + "\t extracted: " + j.strip() + "\t id: " + k
        else:
            print "fp, brand: " + i.strip() + ", " + "\t extracted: " + j.strip() + "\t id: " + k
            false_positive += 1

    print "precision: ", true_positive * 1.0 / (true_positive + false_positive), \
        "recall: ", true_positive * 1.0 / (true_positive + false_negative)

    print "true positive: ", true_positive, " true negative: ", true_negative
    print "false positive: ", false_positive, " false negative: ", false_negative


# --------------------------------------------------------
# Extract brand name from product name.
# Expecting the input data (development) and test files
# containing record in the form of
#
# id?product_name?brand_name
# ...
#
# The second argument corresponds to the dictionary used.
# Using longest substring match to extract brand name.
# --------------------------------------------------------
def extractor(datafile, dictfile, testfile):
    data = list()
    with open(datafile, 'r') as f:
        for line in f:
            data.append(line.rstrip('\n').split('?'))

    dict = list()
    with open(dictfile, 'r') as f:
        for line in f:
            dict.append(line.rstrip('1234567890\n').rstrip())

    test = list()
    with open(testfile, 'r') as f:
        for line in f:
            test.append(line.rstrip('\n').split('?'))

    # merge brand names in the development set with the given dictionary
    merge(dict, data)

    # for brand in dict:
    #     print brand

    # do the actual testing
    extracted_brand = list()
    for rec in test:
        match = ""
        index = len(rec[PROD])
        product_name = " " + rec[PROD].lower() + " "
        real_product_name = " " + rec[PROD] + " "
        for brand in dict:
            guess_brand = " " + brand.lower().strip() + " "

            try:
                pos = product_name.index(guess_brand)
            except ValueError:
                pos = index + 1

            id = get_id([rec])

            # if (brand.lower() in rec[PROD].lower()) and (len(match) < len(brand)):  # initial rule
            # if (guess_brand in product_name) and (len(match) < len(brand)):  #second attempt
            if (pos < index) or ((pos == index) and (len(match) < len(brand))):  # third try
                index = pos
                #match = brand
                match = real_product_name[pos + 1:(pos + 1) + len(guess_brand) - 2]

        # try finding suffixes
        for suffix in SUFFIXES:
            if (" " + match + " " + suffix + " ").lower() in product_name:
                match = match + " " + suffix
                break

        extracted_brand.append(match)

    # calculate precision and recall
    compute_pr(get_id(test), get_truth(test), extracted_brand)


if __name__ == '__main__':
    # extractor(sys.argv[1], sys.argv[2], sys.argv[3])
    extractor('set_I.txt', 'elec_brand_dic.txt', 'set_J.txt')
